@@ -1,4 +1,5 @@
 {-# LANGUAGE ConstraintKinds
+           , EmptyDataDecls
            , FlexibleContexts
            , FlexibleInstances
            , FunctionalDependencies
@@ -29,6 +30,7 @@ module Database.Esqueleto.Internal.Sql
   , unsafeSqlBinOp
   , unsafeSqlValue
   , unsafeSqlFunction
+  , unsafeSqlExtractSubField
   , UnsafeSqlFunctionArgument
   , rawSelectSource
   , runSource
@@ -38,8 +40,9 @@ module Database.Esqueleto.Internal.Sql
   , IdentState
   , initialIdentState
   , IdentInfo
-  , SqlSelect
+  , SqlSelect(..)
   , veryUnsafeCoerceSqlExprValue
+  , veryUnsafeCoerceSqlExprValueList
   ) where
 
 import Control.Applicative (Applicative(..), (<$>), (<$))
@@ -47,15 +50,13 @@ import Control.Arrow ((***), first)
 import Control.Exception (throw, throwIO)
 import Control.Monad ((>=>), ap, void, MonadPlus(..))
 import Control.Monad.IO.Class (MonadIO(..))
-import Control.Monad.Logger (MonadLogger)
 import Control.Monad.Trans.Class (lift)
-import Control.Monad.Trans.Resource (MonadResourceBase)
+import Control.Monad.Trans.Resource (MonadResource)
 import Data.Int (Int64)
 import Data.List (intersperse)
 import Data.Monoid (Monoid(..), (<>))
 import Data.Proxy (Proxy(..))
 import Database.Esqueleto.Internal.PersistentImport
-import qualified Control.Monad.Trans.Reader as R
 import qualified Control.Monad.Trans.State as S
 import qualified Control.Monad.Trans.Writer as W
 import qualified Data.Conduit as C
@@ -64,7 +65,6 @@ import qualified Data.HashSet as HS
 import qualified Data.Text as T
 import qualified Data.Text.Lazy as TL
 import qualified Data.Text.Lazy.Builder as TLB
-import qualified Data.Text.Lazy.Builder.Int as TLBI
 
 import Database.Esqueleto.Internal.Language
 
@@ -255,6 +255,7 @@ data SqlExpr a where
 
   -- A 'SqlExpr' accepted only by 'orderBy'.
   EOrderBy :: OrderByType -> SqlExpr (Value a) -> SqlExpr OrderBy
+  EOrderRandom :: SqlExpr OrderBy
 
   -- A 'SqlExpr' accepted only by 'set'.
   ESet :: (SqlExpr (Entity val) -> SqlExpr (Value ())) -> SqlExpr (Update val)
@@ -264,6 +265,10 @@ data SqlExpr a where
 
   -- Used by 'insertSelect'.
   EInsert  :: Proxy a -> (IdentInfo -> (TLB.Builder, [PersistValue])) -> SqlExpr (Insertion a)
+  EInsertFinal :: PersistEntity a => SqlExpr (Insertion a) -> SqlExpr InsertFinal
+
+-- | Phantom type used to mark a @INSERT INTO@ query.
+data InsertFinal
 
 data NeedParens = Parens | Never
 
@@ -317,6 +322,8 @@ instance Esqueleto SqlQuery SqlExpr SqlBackend where
   asc  = EOrderBy ASC
   desc = EOrderBy DESC
 
+  rand = EOrderRandom
+
   limit  n = Q $ W.tell mempty { sdLimitClause = Limit (Just n) Nothing  }
   offset n = Q $ W.tell mempty { sdLimitClause = Limit Nothing  (Just n) }
 
@@ -417,7 +424,7 @@ setAux field mkVal = ESet $ \ent -> unsafeSqlBinOp " = " name (mkVal ent)
   where name = ERaw Never $ \info -> (fieldName info field, mempty)
 
 sub :: PersistField a => Mode -> SqlQuery (SqlExpr (Value a)) -> SqlExpr (Value a)
-sub mode query = ERaw Parens $ \info -> toRawSql mode pureQuery info query
+sub mode query = ERaw Parens $ \info -> toRawSql mode info query
 
 fromDBName :: IdentInfo -> DBName -> TLB.Builder
 fromDBName (conn, _) = TLB.fromText . connEscapeName conn
@@ -475,6 +482,19 @@ unsafeSqlFunction name arg =
           uncommas' $ map (\(ERaw _ f) -> f info) $ toArgList arg
     in (name <> parens argsTLB, argsVals)
 
+-- | (Internal) An unsafe SQL function to extract a subfield from a compound
+-- field, e.g. datetime. See 'unsafeSqlBinOp' for warnings.
+--
+-- Since: 1.3.6.
+unsafeSqlExtractSubField :: UnsafeSqlFunctionArgument a =>
+                     TLB.Builder -> a -> SqlExpr (Value b)
+unsafeSqlExtractSubField subField arg =
+  ERaw Never $ \info ->
+    let (argsTLB, argsVals) =
+          uncommas' $ map (\(ERaw _ f) -> f info) $ toArgList arg
+    in ("EXTRACT" <> parens (subField <> " FROM " <> argsTLB), argsVals)
+
+
 class UnsafeSqlFunctionArgument a where
   toArgList :: a -> [SqlExpr (Value ())]
 instance (a ~ Value b) => UnsafeSqlFunctionArgument (SqlExpr a) where
@@ -499,6 +519,7 @@ instance ( UnsafeSqlFunctionArgument a
   toArgList = toArgList . from4
 
 
+
 -- | (Internal) Coerce a value's type from 'SqlExpr (Value a)' to
 -- 'SqlExpr (Value b)'.  You should /not/ use this function
 -- unless you know what you're doing!
@@ -520,21 +541,21 @@ veryUnsafeCoerceSqlExprValueList EEmptyList =
 -- | (Internal) Execute an @esqueleto@ @SELECT@ 'SqlQuery' inside
 -- @persistent@'s 'SqlPersistT' monad.
 rawSelectSource :: ( SqlSelect a r
-                   , MonadLogger m
-                   , MonadResourceBase m )
+                   , MonadResource m
+                   , MonadSqlPersist m )
                  => Mode
                  -> SqlQuery a
-                 -> SqlPersistT m (C.Source (C.ResourceT (SqlPersistT m)) r)
+                 -> m (C.Source m r)
 rawSelectSource mode query = src
     where
       src = do
-        conn <- SqlPersistT R.ask
+        conn <- askSqlConn
         return $ run conn C.$= massage
 
       run conn =
         uncurry rawQuery $
         first builderToText $
-        toRawSql mode pureQuery (conn, initialIdentState) query
+        toRawSql mode (conn, initialIdentState) query
 
       massage = do
         mrow <- C.await
@@ -549,10 +570,10 @@ rawSelectSource mode query = src
 -- | Execute an @esqueleto@ @SELECT@ query inside @persistent@'s
 -- 'SqlPersistT' monad and return a 'C.Source' of rows.
 selectSource :: ( SqlSelect a r
-                , MonadLogger m
-                , MonadResourceBase m )
+                , MonadResource m
+                , MonadSqlPersist m )
              => SqlQuery a
-             -> SqlPersistT m (C.Source (C.ResourceT (SqlPersistT m)) r)
+             -> m (C.Source m r)
 selectSource = rawSelectSource SELECT
 
 
@@ -598,9 +619,9 @@ selectSource = rawSelectSource SELECT
 -- function composition that the @p@ inside the query is of type
 -- @SqlExpr (Entity Person)@.
 select :: ( SqlSelect a r
-          , MonadLogger m
-          , MonadResourceBase m )
-       => SqlQuery a -> SqlPersistT m [r]
+          , MonadResource m
+          , MonadSqlPersist m )
+       => SqlQuery a -> m [r]
 select = selectSource >=> runSource
 
 
@@ -609,27 +630,27 @@ select = selectSource >=> runSource
 -- rows.
 selectDistinctSource
   :: ( SqlSelect a r
-     , MonadLogger m
-     , MonadResourceBase m )
+     , MonadResource m
+     , MonadSqlPersist m )
   => SqlQuery a
-  -> SqlPersistT m (C.Source (C.ResourceT (SqlPersistT m)) r)
+  -> m (C.Source m r)
 selectDistinctSource = rawSelectSource SELECT_DISTINCT
 
 
 -- | Execute an @esqueleto@ @SELECT DISTINCT@ query inside
 -- @persistent@'s 'SqlPersistT' monad and return a list of rows.
 selectDistinct :: ( SqlSelect a r
-                  , MonadLogger m
-                  , MonadResourceBase m )
-               => SqlQuery a -> SqlPersistT m [r]
+                  , MonadResource m
+                  , MonadSqlPersist m )
+               => SqlQuery a -> m [r]
 selectDistinct = selectDistinctSource >=> runSource
 
 
 -- | (Internal) Run a 'C.Source' of rows.
-runSource :: MonadResourceBase m =>
-             C.Source (C.ResourceT (SqlPersistT m)) r
-          -> SqlPersistT m [r]
-runSource src = C.runResourceT $ src C.$$ CL.consume
+runSource :: MonadResource m =>
+             C.Source m r
+          -> m [r]
+runSource src = src C.$$ CL.consume
 
 
 ----------------------------------------------------------------------
@@ -637,16 +658,17 @@ runSource src = C.runResourceT $ src C.$$ CL.consume
 
 -- | (Internal) Execute an @esqueleto@ statement inside
 -- @persistent@'s 'SqlPersistT' monad.
-rawEsqueleto :: ( MonadLogger m
-              , MonadResourceBase m )
+rawEsqueleto :: ( MonadResource m
+                , MonadSqlPersist m
+                , SqlSelect a r )
            => Mode
-           -> SqlQuery ()
-           -> SqlPersistT m Int64
+           -> SqlQuery a
+           -> m Int64
 rawEsqueleto mode query = do
-  conn <- SqlPersistT R.ask
+  conn <- askSqlConn
   uncurry rawExecuteCount $
     first builderToText $
-    toRawSql mode pureQuery (conn, initialIdentState) query
+    toRawSql mode (conn, initialIdentState) query
 
 
 -- | Execute an @esqueleto@ @DELETE@ query inside @persistent@'s
@@ -671,18 +693,18 @@ rawEsqueleto mode query = do
 -- from $ \\(appointment :: SqlExpr (Entity Appointment)) ->
 -- return ()
 -- @
-delete :: ( MonadLogger m
-          , MonadResourceBase m )
+delete :: ( MonadResource m
+          , MonadSqlPersist m )
        => SqlQuery ()
-       -> SqlPersistT m ()
+       -> m ()
 delete = void . deleteCount
 
 
 -- | Same as 'delete', but returns the number of rows affected.
-deleteCount :: ( MonadLogger m
-               , MonadResourceBase m )
+deleteCount :: ( MonadResource m
+               , MonadSqlPersist m )
             => SqlQuery ()
-            -> SqlPersistT m Int64
+            -> m Int64
 deleteCount = rawEsqueleto DELETE
 
 
@@ -698,23 +720,42 @@ deleteCount = rawEsqueleto DELETE
 -- set p [ PersonAge =. just (val thisYear) -. p ^. PersonBorn ]
 -- where_ $ isNull (p ^. PersonAge)
 -- @
-update :: ( MonadLogger m
-          , MonadResourceBase m
+update :: ( MonadResource m
+          , MonadSqlPersist m
           , SqlEntity val )
        => (SqlExpr (Entity val) -> SqlQuery ())
-       -> SqlPersistT m ()
+       -> m ()
 update = void . updateCount
 
 
 -- | Same as 'update', but returns the number of rows affected.
-updateCount :: ( MonadLogger m
-               , MonadResourceBase m
+updateCount :: ( MonadResource m
+               , MonadSqlPersist m
                , SqlEntity val )
             => (SqlExpr (Entity val) -> SqlQuery ())
-            -> SqlPersistT m Int64
+            -> m Int64
 updateCount = rawEsqueleto UPDATE . from
 
 
+-- | Insert a 'PersistField' for every selected value.
+insertSelect :: ( MonadResource m
+                , MonadSqlPersist m
+                , PersistEntity a )
+             => SqlQuery (SqlExpr (Insertion a)) -> m ()
+insertSelect =
+  void . rawEsqueleto (INSERT_INTO SELECT) . fmap EInsertFinal
+
+
+-- | Insert a 'PersistField' for every unique selected value.
+insertSelectDistinct
+  :: ( MonadResource m
+     , MonadSqlPersist m
+     , PersistEntity a )
+  => SqlQuery (SqlExpr (Insertion a)) -> m ()
+insertSelectDistinct =
+  void . rawEsqueleto (INSERT_INTO SELECT_DISTINCT) . fmap EInsertFinal
+
+
 ----------------------------------------------------------------------
 
 
@@ -730,8 +771,8 @@ builderToText = TL.toStrict . TLB.toLazyTextWith defaultChunkSize
 -- @esqueleto@, instead of manually using this function (which is
 -- possible but tedious), you may just turn on query logging of
 -- @persistent@.
-toRawSql :: SqlSelect a r => Mode -> QueryType a -> IdentInfo -> SqlQuery a -> (TLB.Builder, [PersistValue])
-toRawSql mode qt (conn, firstIdentState) query =
+toRawSql :: SqlSelect a r => Mode -> IdentInfo -> SqlQuery a -> (TLB.Builder, [PersistValue])
+toRawSql mode (conn, firstIdentState) query =
   let ((ret, sd), finalIdentState) =
         flip S.runState firstIdentState $
         W.runWriterT $
@@ -749,36 +790,27 @@ toRawSql mode qt (conn, firstIdentState) query =
       -- appear on the expressions below.
       info = (conn, finalIdentState)
   in mconcat
-      [ makeInsert  qt ret
-      , makeSelect  info mode ret
-      , makeFrom    info mode fromClauses
-      , makeSet     info setClauses
-      , makeWhere   info whereClauses
-      , makeGroupBy info groupByClause
-      , makeHaving  info havingClause
-      , makeOrderBy info orderByClauses
-      , makeLimit   info limitClause
+      [ makeInsertInto info mode ret
+      , makeSelect     info mode ret
+      , makeFrom       info mode fromClauses
+      , makeSet        info setClauses
+      , makeWhere      info whereClauses
+      , makeGroupBy    info groupByClause
+      , makeHaving     info havingClause
+      , makeOrderBy    info orderByClauses
+      , makeLimit      info limitClause orderByClauses
       ]
 
 
 -- | (Internal) Mode of query being converted by 'toRawSql'.
-data Mode = SELECT | SELECT_DISTINCT | DELETE | UPDATE
+data Mode =
+    SELECT
+  | SELECT_DISTINCT
+  | DELETE
+  | UPDATE
+  | INSERT_INTO Mode
+    -- ^ 'Mode' should be either 'SELECT' or 'SELECT_DISTINCT'.
 
-newtype QueryType a = QueryType { unQueryType :: a -> TLB.Builder }
-
-pureQuery :: QueryType a
-pureQuery = QueryType (const mempty)
-
-insertQuery :: PersistEntity a => QueryType (SqlExpr (Insertion a))
-insertQuery = QueryType $ \(EInsert p _)->
-    let def = entityDef p
-        unName = TLB.fromText . unDBName
-        fields = uncommas $ map (unName . fieldDB) (entityFields def)
-        table = unName . entityDB . entityDef $ p
-    in "INSERT INTO " <> table <> parens fields <> "\n"
-
-makeInsert :: QueryType a -> a -> (TLB.Builder, [PersistValue])
-makeInsert q a = (unQueryType q a, [])
 
 uncommas :: [TLB.Builder] -> TLB.Builder
 uncommas = mconcat . intersperse ", " . filter (/= mempty)
@@ -787,14 +819,21 @@ uncommas' :: Monoid a => [(TLB.Builder, a)] -> (TLB.Builder, a)
 uncommas' = (uncommas *** mconcat) . unzip
 
 
+makeInsertInto :: SqlSelect a r => IdentInfo -> Mode -> a -> (TLB.Builder, [PersistValue])
+makeInsertInto info (INSERT_INTO _) ret = sqlInsertInto info ret
+makeInsertInto _    _               _   = mempty
+
+
 makeSelect :: SqlSelect a r => IdentInfo -> Mode -> a -> (TLB.Builder, [PersistValue])
-makeSelect info mode ret =
-  case mode of
-    SELECT          -> withCols "SELECT "
-    SELECT_DISTINCT -> withCols "SELECT DISTINCT "
-    DELETE          -> plain "DELETE "
-    UPDATE          -> plain "UPDATE "
+makeSelect info mode_ ret = process mode_
   where
+    process mode =
+      case mode of
+        SELECT            -> withCols "SELECT "
+        SELECT_DISTINCT   -> withCols "SELECT DISTINCT "
+        DELETE            -> plain "DELETE "
+        UPDATE            -> plain "UPDATE "
+        INSERT_INTO mode' -> process mode'
     withCols v = first (v <>) (sqlSelectCols info ret)
     plain    v = (v, [])
 
@@ -866,27 +905,19 @@ makeOrderBy :: IdentInfo -> [OrderByClause] -> (TLB.Builder, [PersistValue])
 makeOrderBy _    [] = mempty
 makeOrderBy info os = first ("\nORDER BY " <>) $ uncommas' (map mk os)
   where
+    mk :: OrderByClause -> (TLB.Builder, [PersistValue])
     mk (EOrderBy t (ERaw p f)) = first ((<> orderByType t) . parensM p) (f info)
+    mk EOrderRandom = first ((<> "RANDOM()")) mempty
     orderByType ASC  = " ASC"
     orderByType DESC = " DESC"
 
 
-makeLimit :: IdentInfo -> LimitClause -> (TLB.Builder, [PersistValue])
-makeLimit _    (Limit Nothing Nothing)  = mempty
-makeLimit _    (Limit Nothing (Just 0)) = mempty
-makeLimit info (Limit ml      mo)       = (ret, mempty)
-  where
-    ret = TLB.singleton '\n' <> (limitTLB <> offsetTLB)
-
-    limitTLB =
-      case ml of
-        Just l  -> "LIMIT " <> TLBI.decimal l
-        Nothing -> TLB.fromText (connNoLimit $ fst info)
-
-    offsetTLB =
-      case mo of
-        Just o  -> " OFFSET " <> TLBI.decimal o
-        Nothing -> mempty
+makeLimit :: IdentInfo -> LimitClause -> [OrderByClause] -> (TLB.Builder, [PersistValue])
+makeLimit (conn,_) (Limit ml mo) orderByClauses =
+  let limitRaw = connLimitOffset conn (v ml, v mo) hasOrderClause "\n"
+      hasOrderClause = not (null orderByClauses)
+      v = maybe 0 fromIntegral
+  in (TLB.fromText limitRaw, mempty)
 
 
 parens :: TLB.Builder -> TLB.Builder
@@ -914,14 +945,25 @@ class SqlSelect a r | a -> r, r -> a where
   -- | Transform a row of the result into the data type.
   sqlSelectProcessRow :: [PersistValue] -> Either T.Text r
 
+  -- | Create @INSERT INTO@ clause instead.
+  sqlInsertInto :: IdentInfo -> a -> (TLB.Builder, [PersistValue])
+  sqlInsertInto = error "Type does not support sqlInsertInto."
 
--- | You may return an insertion of some PersistEntity
-instance PersistEntity a => SqlSelect (SqlExpr (Insertion a)) (Insertion a) where
-  sqlSelectCols info (EInsert _ f) = f info
-  sqlSelectColCount = const 0
+
+-- | @INSERT INTO@ hack.
+instance SqlSelect (SqlExpr InsertFinal) InsertFinal where
+  sqlInsertInto info (EInsertFinal (EInsert p _)) =
+    let fields = uncommas $
+                 map (fromDBName info . fieldDB) $
+                 entityFields $
+                 entityDef p
+        table  = fromDBName info . entityDB . entityDef $ p
+    in ("INSERT INTO " <> table <> parens fields <> "\n", [])
+  sqlSelectCols info (EInsertFinal (EInsert _ f)) = f info
+  sqlSelectColCount   = const 0
   sqlSelectProcessRow = const (Right (error msg))
     where
-      msg = "sqlSelectProcessRow/SqlSelect (SqlExpr (Insertion a)) (Insertion a): never here"
+      msg = "sqlSelectProcessRow/SqlSelect/InsertionFinal: never here"
 
 
 -- | Not useful for 'select', but used for 'update' and 'delete'.
@@ -1470,22 +1512,3 @@ from16P = const Proxy
 
 to16 :: ((a,b),(c,d),(e,f),(g,h),(i,j),(k,l),(m,n),(o,p)) -> (a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p)
 to16 ((a,b),(c,d),(e,f),(g,h),(i,j),(k,l),(m,n),(o,p)) = (a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p)
-
-
--- | Insert a 'PersistField' for every selected value.
-insertSelect :: (MonadLogger m, MonadResourceBase m, SqlSelect (SqlExpr (Insertion a)) r, PersistEntity a) =>
-  SqlQuery (SqlExpr (Insertion a)) -> SqlPersistT m ()
-insertSelect = insertGeneralSelect SELECT
-
-
--- | Insert a 'PersistField' for every unique selected value.
-insertSelectDistinct :: (MonadLogger m, MonadResourceBase m, SqlSelect (SqlExpr (Insertion a)) r, PersistEntity a) =>
-  SqlQuery (SqlExpr (Insertion a)) -> SqlPersistT m ()
-insertSelectDistinct = insertGeneralSelect SELECT_DISTINCT
-
-
-insertGeneralSelect :: (MonadLogger m, MonadResourceBase m, SqlSelect (SqlExpr (Insertion a)) r, PersistEntity a) =>
-  Mode -> SqlQuery (SqlExpr (Insertion a)) -> SqlPersistT m ()
-insertGeneralSelect mode query = do
-  conn <- SqlPersistT R.ask
-  uncurry rawExecute $ first builderToText $ toRawSql mode insertQuery (conn, initialIdentState) query
