@@ -10,6 +10,7 @@
            , TemplateHaskell
            , TypeFamilies
            , ScopedTypeVariables
+           , CPP
  #-}
 module Main (main) where
 
@@ -19,12 +20,24 @@ import Control.Monad.IO.Class (MonadIO(liftIO))
 import Control.Monad.Logger (MonadLogger(..), runStderrLoggingT, runNoLoggingT)
 import Control.Monad.Trans.Control (MonadBaseControl(..))
 import Database.Esqueleto
+#if   defined (WITH_POSTGRESQL)
+import Database.Persist.Postgresql (withPostgresqlConn)
+#elif defined (WITH_MYSQL)
+import Database.Persist.MySQL ( withMySQLConn
+                              , connectHost
+                              , connectDatabase
+                              , connectUser
+                              , connectPassword
+                              , defaultConnectInfo)
+#else
 import Database.Persist.Sqlite (withSqliteConn)
+#endif
 import Database.Persist.TH
 import Test.Hspec
 
-import qualified Data.Conduit as C
+import qualified Control.Monad.Trans.Resource as R
 import qualified Data.Set as S
+import qualified Data.List as L
 
 
 -- Test schema
@@ -43,6 +56,10 @@ share [mkPersist sqlSettings, mkMigrate "migrateAll"] [persistUpperCase|
     deriving Eq Show
 |]
 
+-- | this could be achieved with S.fromList, but not all lists
+--   have Ord instances
+sameElementsAs :: Eq a => [a] -> [a] -> Bool
+sameElementsAs l1 l2 = null (l1 L.\\ l2)
 
 main :: IO ()
 main = do
@@ -96,17 +113,17 @@ main = do
           ret <- select $
                  from $ \(person1, person2) ->
                  return (person1, person2)
-          liftIO $ ret `shouldBe` [ (p1e, p1e)
-                                  , (p1e, p2e)
-                                  , (p2e, p1e)
-                                  , (p2e, p2e) ]
+          liftIO $ ret `shouldSatisfy` sameElementsAs [ (p1e, p1e)
+                                                      , (p1e, p2e)
+                                                      , (p2e, p1e)
+                                                      , (p2e, p2e) ]
 
       it "works for a self-join via sub_select" $
         run $ do
           p1k <- insert p1
           p2k <- insert p2
-          f1k <- insert (Follow p1k p2k)
-          f2k <- insert (Follow p2k p1k)
+          _f1k <- insert (Follow p1k p2k)
+          _f2k <- insert (Follow p2k p1k)
           ret <- select $
                  from $ \followA -> do
                  let subquery =
@@ -121,8 +138,8 @@ main = do
         run $ do
           p1k <- insert p1
           p2k <- insert p2
-          f1k <- insert (Follow p1k p2k)
-          f2k <- insert (Follow p2k p1k)
+          _f1k <- insert (Follow p1k p2k)
+          _f2k <- insert (Follow p2k p1k)
           ret <- select $
                  from $ \followA -> do
                  where_ $ exists $
@@ -149,7 +166,8 @@ main = do
           ret <- select $
                  from $ \(pa, pb) ->
                  return (pa ^. PersonName, pb ^. PersonName)
-          liftIO $ ret `shouldBe` [ (Value (personName p1), Value (personName p1))
+          liftIO $ ret `shouldSatisfy` sameElementsAs
+                                  [ (Value (personName p1), Value (personName p1))
                                   , (Value (personName p1), Value (personName p2))
                                   , (Value (personName p2), Value (personName p1))
                                   , (Value (personName p2), Value (personName p2)) ]
@@ -306,7 +324,13 @@ main = do
           ret <- select $
                  from $ \p->
                  return $ joinV $ sum_ (p ^. PersonAge)
+#if   defined(WITH_POSTGRESQL)
+          liftIO $ ret `shouldBe` [ Value $ Just (36 + 17 + 17 :: Rational ) ]
+#elif defined(WITH_MYSQL)
+          liftIO $ ret `shouldBe` [ Value $ Just (36 + 17 + 17 :: Double ) ]
+#else
           liftIO $ ret `shouldBe` [ Value $ Just (36 + 17 + 17 :: Int) ]
+#endif
 
       it "works with avg_" $
         run $ do
@@ -343,7 +367,11 @@ main = do
 
       it "works with random_" $
         run $ do
-          ret <- select $ return (random_ :: SqlExpr (Value Int))
+#if defined(WITH_POSTGRESQL) || defined(WITH_MYSQL)
+          _ <- select $ return (random_ :: SqlExpr (Value Double))
+#else
+          _ <- select $ return (random_ :: SqlExpr (Value Int))
+#endif
           return ()
 
       it "works with round_" $
@@ -462,7 +490,13 @@ main = do
                  from $ \p -> do
                  orderBy [asc (p ^. PersonAge), asc (p ^. PersonName)]
                  return p
+          -- in PostgreSQL nulls are bigger than everything
+#ifdef WITH_POSTGRESQL
+          liftIO $ ret `shouldBe` [ p4e, p3e, p1e , p2e ]
+#else
+          -- in SQLite and MySQL, its the reverse
           liftIO $ ret `shouldBe` [ p2e, p4e, p3e, p1e ]
+#endif
 
       it "works with one ASC and one DESC field" $
         run $ do
@@ -474,7 +508,11 @@ main = do
                  from $ \p -> do
                  orderBy [desc (p ^. PersonAge), asc (p ^. PersonName)]
                  return p
+#ifdef WITH_POSTGRESQL
+          liftIO $ ret `shouldBe` [ p2e, p1e, p4e, p3e ]
+#else
           liftIO $ ret `shouldBe` [ p1e, p4e, p3e, p2e ]
+#endif
 
       it "works with a sub_select" $
         run $ do
@@ -492,10 +530,10 @@ main = do
 
       it "works with asc random_" $
         run $ do
-          p1e <- insert' p1
-          p2e <- insert' p2
-          p3e <- insert' p3
-          p4e <- insert' p4
+          _p1e <- insert' p1
+          _p2e <- insert' p2
+          _p3e <- insert' p3
+          _p4e <- insert' p4
           rets <-
             fmap S.fromList $
             replicateM 11 $
@@ -578,10 +616,27 @@ main = do
                  from $ \p -> do
                  orderBy [ asc (p ^. PersonName), asc (p ^. PersonAge) ]
                  return p
+          -- PostgreSQL: nulls are bigger than data, and update returns
+          --             matched rows, not actually changed rows.
+#if   defined(WITH_POSTGRESQL)
+          liftIO $ n `shouldBe` 2
+          liftIO $ ret `shouldBe` [ Entity p1k (Person anon (Just 73))
+                                  , Entity p2k (Person anon Nothing)
+                                  , Entity p3k p3 ]
+          -- MySQL: nulls appear first, and update returns actual number
+          --        of changed rows
+#elif defined(WITH_MYSQL)
+          liftIO $ n `shouldBe` 1
+          liftIO $ ret `shouldBe` [ Entity p2k (Person anon Nothing)
+                                  , Entity p1k (Person anon (Just 73))
+                                  , Entity p3k p3 ]
+#else
+          -- SQLite: nulls appear first, update returns matched rows.
           liftIO $ n `shouldBe` 2
           liftIO $ ret `shouldBe` [ Entity p2k (Person anon Nothing)
                                   , Entity p1k (Person anon (Just 73))
                                   , Entity p3k p3 ]
+#endif
 
       it "works with a subexpression having COUNT(*)" $
         run $ do
@@ -625,7 +680,7 @@ main = do
       it "GROUP BY works with HAVING" $
         run $ do
           p1k <- insert p1
-          p2k <- insert p2
+          _p2k <- insert p2
           p3k <- insert p3
           replicateM_ 3 (insert $ BlogPost "" p1k)
           replicateM_ 7 (insert $ BlogPost "" p3k)
@@ -645,7 +700,7 @@ main = do
         run $ do
           p1k <- insert p1
           p2k <- insert p2
-          p3k <- insert p3
+          _p3k <- insert p3
           ret <- select $
                  from $ \p -> do
                  where_ (p ^. PersonName `in_` valList (personName <$> [p1, p2]))
@@ -655,9 +710,9 @@ main = do
 
       it "IN works for valList (null list)" $
         run $ do
-          p1k <- insert p1
-          p2k <- insert p2
-          p3k <- insert p3
+          _p1k <- insert p1
+          _p2k <- insert p2
+          _p3k <- insert p3
           ret <- select $
                  from $ \p -> do
                  where_ (p ^. PersonName `in_` valList [])
@@ -667,7 +722,7 @@ main = do
       it "IN works for subList_select" $
         run $ do
           p1k <- insert p1
-          p2k <- insert p2
+          _p2k <- insert p2
           p3k <- insert p3
           _ <- insert (BlogPost "" p1k)
           _ <- insert (BlogPost "" p3k)
@@ -701,7 +756,7 @@ main = do
       it "EXISTS works for subList_select" $
         run $ do
           p1k <- insert p1
-          p2k <- insert p2
+          _p2k <- insert p2
           p3k <- insert p3
           _ <- insert (BlogPost "" p1k)
           _ <- insert (BlogPost "" p3k)
@@ -737,9 +792,29 @@ main = do
           _ <- insert p3
           insertSelect $ from $ \p -> do
             return $ BlogPost <# val "FakePost" <&> (p ^. PersonId)
-          ret <- select $ from (\(b::(SqlExpr (Entity BlogPost))) -> return countRows)
+          ret <- select $ from (\(_::(SqlExpr (Entity BlogPost))) -> return countRows)
           liftIO $ ret `shouldBe` [Value (3::Int)]
 
+    describe "rand works" $ do
+      it "returns result in random order" $
+        run $ do
+          replicateM_ 20 $ do
+            _ <- insert p1
+            _ <- insert p2
+            _ <- insert p3
+            _ <- insert p4
+            _ <- insert $ Person "Jane"  Nothing
+            _ <- insert $ Person "Mark"  Nothing
+            _ <- insert $ Person "Sarah" Nothing
+            insert $ Person "Paul"  Nothing
+          ret1 <- fmap (map unValue) $ select $ from $ \p -> do
+                    orderBy [rand]
+                    return (p ^. PersonId)
+          ret2 <- fmap (map unValue) $ select $ from $ \p -> do
+                    orderBy [rand]
+                    return (p ^. PersonId)
+
+          liftIO $ (ret1 == ret2) `shouldBe` False
 
 ----------------------------------------------------------------------
 
@@ -753,10 +828,24 @@ insert' v = flip Entity v <$> insert v
 
 
 type RunDbMonad m = ( MonadBaseControl IO m, MonadIO m, MonadLogger m
-                    , C.MonadUnsafeIO m, C.MonadThrow m )
+                    , R.MonadThrow m )
+
+#if defined (WITH_POSTGRESQL) || defined (WITH_MYSQL)
+-- With SQLite and in-memory databases, a separate connection implies a
+-- separate database. With 'actual databases', the data is persistent and
+-- thus must be cleaned after each test.
+-- TODO: there is certainly a better way...
+cleanDB
+  :: (forall m. RunDbMonad m
+  => SqlPersistT (R.ResourceT m) ())
+cleanDB = do
+  delete $ from $ \(blogpost :: SqlExpr (Entity BlogPost))-> return ()
+  delete $ from $ \(follow   :: SqlExpr (Entity Follow))  -> return ()
+  delete $ from $ \(person   :: SqlExpr (Entity Person))  -> return ()
+#endif
 
 
-run, runSilent, runVerbose :: (forall m. RunDbMonad m => SqlPersistT (C.ResourceT m) a) -> IO a
+run, runSilent, runVerbose :: (forall m. RunDbMonad m => SqlPersistT (R.ResourceT m) a) -> IO a
 runSilent  act = runNoLoggingT     $ run_worker act
 runVerbose act = runStderrLoggingT $ run_worker act
 run =
@@ -769,9 +858,24 @@ verbose :: Bool
 verbose = True
 
 
-run_worker :: RunDbMonad m => SqlPersistT (C.ResourceT m) a -> m a
-run_worker =
-  C.runResourceT .
+run_worker :: RunDbMonad m => SqlPersistT (R.ResourceT m) a -> m a
+run_worker act =
+  R.runResourceT .
+#if defined(WITH_POSTGRESQL)
+  withPostgresqlConn "host=localhost port=5432 user=test dbname=test" .
+#elif defined (WITH_MYSQL)
+  withMySQLConn defaultConnectInfo
+    { connectHost     = "localhost"
+    , connectUser     = "test"
+    , connectPassword = "test"
+    , connectDatabase = "test"
+    } .
+#else
   withSqliteConn ":memory:" .
+#endif
   runSqlConn .
-  (runMigrationSilent migrateAll >>)
+#if defined (WITH_POSTGRESQL) || defined (WITH_MYSQL)
+  (runMigrationSilent migrateAll >>) $ (cleanDB >> act)
+#else
+  (runMigrationSilent migrateAll >>) $ act
+#endif
